/*
 * output_helper.c
 * CS3243 Homework 2I: Light Up
 *
 * Sample C code framework for an output helper for Light Up.
 */

#include <stdio.h>
#include <stdlib.h>

/*
 * Global variables.
 */

int g_numrows;
int g_numcols;
char** g_grid;

/*
 * Reads input file subject to specifications on: 
 * http://www.comp.nus.edu.sg/~kanmy/courses/3243_2007/hw-lightup.html
 */

void readInputFile(char* inputfile) {
	FILE* fp;
	char* buf;
	int r;
	int c;

	/* Open input file. */
	fp = fopen(inputfile, "r");
	if (fp == NULL) {
		fprintf(stderr, "Error: Unable to open input file %s\n", inputfile);
		exit(1);
	}

	/* Read input file and construct grid. */
	fscanf(fp, "%d %d", &g_numrows, &g_numcols);
	g_grid = (char**)malloc(sizeof(char*) * g_numrows);
	for (r = 0; r < g_numrows; r++)
		g_grid[r] = (char*)malloc(sizeof(char) * g_numcols);
	buf = (char*)malloc((g_numcols + 1) * sizeof(char));
	for (r = 0; r < g_numrows; r++) {
		fscanf(fp, "%s", buf);
		for (c = 0; c < g_numcols; c++)
			g_grid[r][c] = buf[c];
	}

	/* Close file. */
	fclose(fp);
}

/*
 * Reads the solution file generated by the Prolog run.
 */

void readPrologFile(char* prologfile) {
	FILE* fp;
	int row;
	int col;

	/* Open Prolog file. */
	fp = fopen(prologfile, "r");
	if (fp == NULL) {
		fprintf(stderr, "Error: Unable to open Prolog solution file %s\n", prologfile);
		exit(1);
	}

	/* Read input file and construct grid. */
	while (fscanf(fp, "%d %d", &row, &col) == 2)
		g_grid[row][col] = 'L';

	/* Close file. */
	fclose(fp);
}

/*
 * Writes the final output file.
 */

void writeOutputFile(char* outputfile) {
	FILE* fp;
	int r;
	int c;

	/* Open output file. */
	fp = fopen(outputfile, "w");
	if (fp == NULL) {
		fprintf(stderr, "Error: Unable to open output file %s\n", outputfile);
		exit(1);
	}

	/* Writes the output grid. */
	for (r = 0; r < g_numrows; r++) {
		for (c = 0; c < g_numcols; c++)
			fprintf(fp, "%c", g_grid[r][c]);
		fprintf(fp, "\n");
	}

	/* Close file. */
	fclose(fp);
}

/*
 * Usage message, run if invoked incorrectly.
 */

void usage(char* progname) {
	fprintf (stderr, "Usage: %s input_file prolog_file output_file\n", progname);
	exit(1);
}

/*
 * Main function.
 */

int main(int argc, char** argv) {
	/* Check number of parameters. */
	if (argc != 4)
		usage(argv[0]);

	/* Read the grid from the input file. */
	readInputFile(argv[1]);

	/* Read the solution file generated by the Prolog run. */
	readPrologFile(argv[2]);

	/* Write the query file. */
	writeOutputFile(argv[3]);

	return 0;
}

